@@ -1,7 +1,7 @@
 // apt.cc
 //
 //  Copyright 1999-2010 Daniel Burrows
-//  Copyright 2015 Manuel A. Fernandez Montecelo
+//  Copyright 2015-2016 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; see the file COPYING.  If not, write to
-//  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//  Boston, MA 02111-1307, USA.
+//  the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//  Boston, MA 02110-1301, USA.
 //
 //  Handles basic apt bookkeeping.
 
@@ -26,6 +26,7 @@
 #include <loggers.h>
 
 #include "aptitude_resolver_universe.h"
+#include "config_file.h"
 #include "config_signal.h"
 #include "download_queue.h"
 #include "resolver_manager.h"
@@ -41,9 +42,12 @@
 
 #include <generic/util/undo.h>
 
+#include <apt-pkg/acquire.h>
 #include <apt-pkg/aptconfiguration.h>
+#include <apt-pkg/clean.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/depcache.h>
+#include <apt-pkg/dpkgpm.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/init.h>
@@ -51,7 +55,12 @@
 #include <apt-pkg/sourcelist.h>
 #include <apt-pkg/version.h>
 
+#include <boost/filesystem.hpp>
+
+#include <exception>
 #include <fstream>
+#include <regex>
+#include <sstream>
 
 #include <signal.h>
 #include <unistd.h>
@@ -62,6 +71,7 @@ using namespace std;
 using aptitude::Loggers;
 
 namespace cw = cwidget;
+namespace fs = boost::filesystem;
 
 enum interesting_state {uncached = 0, uninteresting, interesting};
 static interesting_state *cached_deps_interesting = NULL;
@@ -87,6 +97,10 @@ sigc::signal0<void> consume_errors;
 
 static string apt_native_arch;
 
+// Access to the download_cache
+std::shared_ptr<aptitude::util::file_cache> download_cache;
+
+
 static void reset_interesting_dep_memoization()
 {
   delete[] cached_deps_interesting;
@@ -104,6 +118,29 @@ bool get_apt_knows_about_rootdir()
   return apt_knows_about_rootdir;
 }
 
+/** Read and check that the config file doesn't contain errors, otherwise exit.
+ *
+ * See #472701 for the motivation.  Upon reading the configuration file
+ * (especially "~/.aptitude/config"), it was writing it back immediately, and
+ * the writing stopped at the point of the reading failure (instead of just
+ * skipping the problematic parts), so part of the previous configuration file
+ * was lost.
+ *
+ * It is safer (and not too onerous) to ask the user to fix the configuration
+ * before continuing, rather than stomping on valid configuration values.
+ */
+void readconfigfile_or_die(Configuration& config, const std::string& path)
+{
+  bool config_ok = ReadConfigFile(config, path);
+
+  if (!config_ok)
+    {
+      _error->Error(_("Configuration file '%s' is not correct, please fix it"), path.c_str());
+      _error->DumpErrors();
+      exit(EXIT_FAILURE);
+    }
+}
+
 void apt_preinit(const char *rootdir)
 {
   logging::LoggerPtr logger(Loggers::getAptitudeAptGlobals());
@@ -148,12 +185,11 @@ void apt_preinit(const char *rootdir)
       user_config->Set("RootDir", rootdir);
     }
 
-  ReadConfigFile(*theme_config, PKGDATADIR "/aptitude-defaults");
+  readconfigfile_or_die(*theme_config, PKGDATADIR "/aptitude-defaults");
 
   pkgInitConfig(*_config);
 
-
-  ReadConfigFile(*_config, PKGDATADIR "/section-descriptions");
+  readconfigfile_or_die(*_config, PKGDATADIR "/section-descriptions");
 
   // TRANSLATORS: Set this string to the name of a configuration
   // file in $pkgdatadir/aptitude that overrides defaults for your
@@ -166,8 +202,10 @@ void apt_preinit(const char *rootdir)
   // "aptitude-defaults.ww".  If you use this mechanism, you should
   // also add your defaults file to pkgdata_DATA in Makefile.am.
   std::string localized_config_name = P_("Localized defaults|");
-  if(localized_config_name.size() > 0)
-    ReadConfigFile(*_config, PKGDATADIR "/" + localized_config_name);
+  if (localized_config_name.size() > 0)
+    {
+      readconfigfile_or_die(*_config, string(PKGDATADIR "/") + localized_config_name);
+    }
 
   pkgInitSystem(*_config, _system);
 
@@ -188,8 +226,8 @@ void apt_preinit(const char *rootdir)
 
   if(!cfgloc.empty() && access(cfgloc.c_str(), R_OK) == 0)
     {
-      ReadConfigFile(*user_config, cfgloc);
-      ReadConfigFile(*_config, cfgloc);
+      readconfigfile_or_die(*user_config, cfgloc);
+      readconfigfile_or_die(*_config, cfgloc);
     }
 
   aptcfg=new signalling_config(user_config, _config, theme_config);
@@ -226,68 +264,26 @@ void apt_preinit(const char *rootdir)
 
 void apt_dumpcfg(const char *root)
 {
-  string cfgloc;
-
-  const char *HOME = getenv("HOME");
-  if(strempty(HOME) == false)
-    {
-      string tmp(HOME);
-      tmp += "/.aptitude";
-      if(access(tmp.c_str(), W_OK) == 0)
-	cfgloc = tmp + "/config";
-      else if(access(tmp.c_str(), R_OK | X_OK) == 0)
-	{
-	  // The squashed-root case.
-	  _error->Error(_("%s is readable but not writable; unable to write configuration file."),
-			tmp.c_str());
-	  return;
-	}
-    }
-
-  if(cfgloc.empty())
-    {
-      cfgloc = get_homedir();
-
-      if(cfgloc.empty())
-	return;
-
-      cfgloc += "/.aptitude";
-
-      if(mkdir(cfgloc.c_str(), 0700)<0 && errno != EEXIST)
-	{
-	  _error->Errno("mkdir", "%s", cfgloc.c_str());
-	  return;
-	}
-
-      cfgloc += "/config";
-    }
-
-  // perhaps should use generic/util/temp.* or be implemented in a better way,
-  // but this is better than what was before (see #764046)
-  string cfgloc_new = cfgloc + ".new-" + std::to_string(getpid());
-
-  ofstream f(cfgloc_new.c_str());
-
-  if(!f)
-    {
-      _error->Errno("apt_init", _("Unable to open %s for writing"), cfgloc_new.c_str());
-      return;
-    }
-
   // Don't write RootDir to the user's configuration file -- it causes
   // horrible confusion.
   std::string rootDir(_config->Find("RootDir", ""));
   _config->Clear("RootDir");
 
-  aptcfg->Dump(f);
+  std::ostringstream content;
+  aptcfg->Dump(content);
 
   _config->Set("RootDir", rootDir);
 
-  f.close();
-
-  if (rename(cfgloc_new.c_str(), cfgloc.c_str()) != 0)
+  try
     {
-      _error->Errno("apt_init", _("Unable to replace %s with new configuration file"), cfgloc.c_str());
+      if ( ! aptitude::apt::ConfigFile::write(content.str()) )
+	{
+	  throw std::runtime_error("");
+	}
+    }
+  catch (...)
+    {
+      _error->Errno("apt_dumpcfg", _("Error saving configuration file"));
       return;
     }
 }
@@ -319,10 +315,11 @@ void apt_revertoptions()
 }
 
 void apt_init(OpProgress *progress_bar, bool do_initselections,
+	      bool operation_needs_lock,
 	      const char *status_fname)
 {
   if(!apt_cache_file)
-    apt_reload_cache(progress_bar, do_initselections, status_fname);
+    apt_reload_cache(progress_bar, do_initselections, operation_needs_lock, status_fname);
 }
 
 void apt_close_cache()
@@ -386,7 +383,9 @@ void apt_close_cache()
 }
 
 void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
-		    const char * status_fname)
+		    bool operation_needs_lock,
+		    const char * status_fname,
+		    bool reset_reinstall)
 {
   logging::LoggerPtr logger(Loggers::getAptitudeAptGlobals());
 
@@ -401,7 +400,6 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
   aptitudeCacheFile *new_file=new aptitudeCacheFile;
 
   LOG_TRACE(logger, "Reading the sources list.");
-
   apt_source_list=new pkgSourceList;
   apt_source_list->ReadMainList();
 
@@ -415,9 +413,10 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
 
   LOG_TRACE(logger, "Opening the apt cache.");
 
-  bool open_failed=!new_file->Open(*progress_bar, do_initselections,
-				   (getuid() == 0) && !simulate,
-				   status_fname)
+  bool open_failed=!new_file->Open(progress_bar, do_initselections,
+				   operation_needs_lock && ((getuid() == 0) && !simulate),
+				   status_fname,
+				   reset_reinstall)
     || _error->PendingError();
 
   if(open_failed && getuid() == 0)
@@ -429,8 +428,9 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
       // Don't discard errors, make sure they get displayed instead.
       consume_errors();
 
-      open_failed=!new_file->Open(*progress_bar, do_initselections,
-				  false, status_fname);
+      open_failed=!new_file->Open(progress_bar, do_initselections,
+				  false, status_fname,
+				  reset_reinstall);
 
       if(open_failed)
 	LOG_ERROR(logger, "Unable to load the apt cache at all; giving up.");
@@ -459,7 +459,13 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
   if(!status_fname && apt_cache_file->is_locked())
     {
       LOG_TRACE(logger, "Trying to save the current selection list.");
-      (*apt_cache_file)->save_selection_list(*progress_bar);
+      (*apt_cache_file)->save_selection_list(progress_bar);
+    }
+
+  // stop here if shutdown is in progress
+  if (shutdown_in_progress)
+    {
+      return;
     }
 
   LOG_TRACE(logger, "Loading the apt package records.");
@@ -468,59 +474,159 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
   // Um, good time to clear our undo info.
   apt_undos->clear_items();
 
-  LOG_TRACE(logger, "Loading task information.");
-  aptitude::apt::load_tasks(*progress_bar);
   LOG_TRACE(logger, "Loading tags.");
   aptitude::apt::load_tags(progress_bar);
 
   LOG_TRACE(logger, "Initializing global dependency resolver manager.");
   resman = new resolver_manager(new_file, imm::map<aptitude_resolver_package, aptitude_resolver_version>());
 
-  LOG_TRACE(logger, "Initializing the download cache.");
-  // Open the download cache.  By default, it goes in
-  // ~/.aptitude/cache; it has 512Kb of in-memory cache and 10MB of
-  // on-disk cache.
-  const char *HOME = getenv("HOME");
-  if(strempty(HOME) == false)
-    {
-      std::string download_cache_file_name = string(HOME) + "/.aptitude/cache";
-      const int download_cache_memory_size =
-	aptcfg->FindI(PACKAGE "::UI::DownloadCache::MemorySize", 512 * 1024);
-      const int download_cache_disk_size   =
-	aptcfg->FindI(PACKAGE "::UI::DownloadCache::DiskSize", 10 * 1024 * 1024);
-      try
-	{
-	  download_cache = aptitude::util::file_cache::create(download_cache_file_name,
-							      download_cache_memory_size,
-							      download_cache_disk_size);
-	}
-      catch(cwidget::util::Exception &ex)
-	{
-	  LOG_WARN(logger,
-		   "Can't open the file cache \""
-		   << download_cache_file_name
-		   << "\": " << ex.errmsg());
-	}
-      catch(std::exception &ex)
-	{
-	  LOG_WARN(logger,
-		   "Can't open the file cache \""
-		   << download_cache_file_name
-		   << "\": " << ex.what());
-	}
-    }
-
   LOG_DEBUG(logger, "Emitting cache_reloaded().");
   cache_reloaded();
-
   LOG_TRACE(logger, "Done emitting cache_reloaded().");
 }
 
+std::shared_ptr<aptitude::util::file_cache> get_download_cache()
+{
+  // return if already initialised
+  if (download_cache)
+    return download_cache;
+
+  logging::LoggerPtr logger(Loggers::getAptitudeAptGlobals());
+
+  LOG_INFO(logger, "Loading download_cache.");
+
+  // Open the download cache.  By default, it goes in
+  // ~/.cache/aptitude/metadata-download; it has 512Kb of in-memory cache and
+  // 10MB of on-disk cache.
+  {
+    // remove old path, if exists, so if config is empty and there are no other
+    // files, ~/.aptitude can also be removed
+    try
+      {
+	const char* env_HOME = getenv("HOME");
+	if ( ! strempty(env_HOME))
+	  {
+	    string old_file = string(env_HOME) + "/.aptitude/cache";
+	    fs::remove(old_file);
+	  }
+      }
+    catch (const fs::filesystem_error& e)
+      {
+	// ignore exceptions, if the file does not exist or we cannot remove it,
+	// it doesn't matter
+      }
+
+    // get xdg_cache_home directory to use
+    const char* env_XDG_CACHE_HOME = getenv("XDG_CACHE_HOME");
+    string xdg_cache_home;
+    if ( ! strempty(env_XDG_CACHE_HOME))
+      {
+	xdg_cache_home = string(env_XDG_CACHE_HOME);
+      }
+    else
+      {
+	const char* env_HOME = getenv("HOME");
+	string home = (! strempty(env_HOME)) ? string(env_HOME) : get_homedir();
+	if (home.empty())
+	  {
+	    _error->Error(_("Could not establish home directory (username: '%s')"), get_username().c_str());
+	  }
+	else if ( ! fs::is_directory(home) )
+	  {
+	    _error->Error(_("Home directory does not exist or is not a directory: '%s')"), home.c_str());
+	  }
+	else
+	  {
+	    xdg_cache_home = home + "/.cache";
+	  }
+      }
+
+    // if directory to be used could be gathered, create the path if needed
+    std::string download_cache_dir;
+    if (!xdg_cache_home.empty())
+      {
+	// if dir does not exist, create default $XDG_CACHE_HOME with the right
+	// permisisons (0700) according to the spec -- see
+	// http://standards.freedesktop.org/basedir-spec/latest/ar01s03.html and
+	// http://standards.freedesktop.org/basedir-spec/latest/ar01s04.html
+	if ( ! fs::is_directory(xdg_cache_home) )
+	  {
+	    mode_t previous_umask = umask(0077);
+
+	    try
+	      {
+		fs::create_directory(xdg_cache_home);
+	      }
+	    catch (const fs::filesystem_error& e)
+	      {
+		_error->Error(_("Could not create directory: %s: %s"), xdg_cache_home.c_str(), e.what());
+	      }
+
+	    umask(previous_umask);
+	  }
+
+	// if the directory exist, continue to the next step
+	if ( fs::is_directory(xdg_cache_home) )
+	  {
+	    download_cache_dir = xdg_cache_home + "/aptitude";
+	  }
+      }
+
+    // if directory to be used could be gathered, create full path if needed,
+    // then assign filename
+    std::string download_cache_file_name;
+    if (!download_cache_dir.empty())
+      {
+	try
+	  {
+	    fs::create_directories(download_cache_dir);
+	    download_cache_file_name = download_cache_dir + "/metadata-download";
+	  }
+	catch (const fs::filesystem_error& e)
+	  {
+	    _error->Error(_("Could not create directories: %s: %s"), download_cache_dir.c_str(), e.what());
+	  }
+      }
+
+    // do create the cache file
+    if (!download_cache_file_name.empty())
+      {
+	const int download_cache_memory_size =
+	  aptcfg->FindI(PACKAGE "::UI::DownloadCache::MemorySize", 512 * 1024);
+	const int download_cache_disk_size   =
+	  aptcfg->FindI(PACKAGE "::UI::DownloadCache::DiskSize", 10 * 1024 * 1024);
+	try
+	  {
+	    download_cache = aptitude::util::file_cache::create(download_cache_file_name,
+								download_cache_memory_size,
+								download_cache_disk_size);
+	  }
+	catch(cwidget::util::Exception &ex)
+	  {
+	    LOG_WARN(logger,
+		     "Can't open the file cache \""
+		     << download_cache_file_name
+		     << "\": " << ex.errmsg());
+	  }
+	catch(std::exception &ex)
+	  {
+	    LOG_WARN(logger,
+		     "Can't open the file cache \""
+		     << download_cache_file_name
+		     << "\": " << ex.what());
+	  }
+      }
+  }
+
+  return download_cache;
+}
+
 void apt_reload_cache(OpProgress *progress_bar, bool do_initselections,
+		      bool operation_needs_lock,
 		      const char * status_fname)
 {
   apt_close_cache();
-  apt_load_cache(progress_bar, do_initselections, status_fname);
+  apt_load_cache(progress_bar, do_initselections, operation_needs_lock, status_fname, false);
 }
 
 void apt_shutdown()
@@ -930,8 +1036,8 @@ pkgCache::DepIterator is_conflicted(const pkgCache::VerIterator &ver,
 
 bool can_remove_autoinstalled(const pkgCache::PkgIterator& pkg,
 			      aptitudeDepCache& cache,
-			      bool follow_recommends,
-			      bool follow_suggests)
+			      bool keep_recommends_installed,
+			      bool keep_suggests_installed)
 {
   // if not valid, consider not-OK -- cannot decide if it's safe
   if (pkg.end())
@@ -962,8 +1068,8 @@ bool can_remove_autoinstalled(const pkgCache::PkgIterator& pkg,
       // consider only these type of dependencies
       if (! ((rev_dep->Type == pkgCache::Dep::Depends) ||
 	     (rev_dep->Type == pkgCache::Dep::PreDepends) ||
-	     (rev_dep->Type == pkgCache::Dep::Recommends && follow_recommends) ||
-	     (rev_dep->Type == pkgCache::Dep::Suggests   && follow_suggests)))
+	     (rev_dep->Type == pkgCache::Dep::Recommends && keep_recommends_installed) ||
+	     (rev_dep->Type == pkgCache::Dep::Suggests   && keep_suggests_installed)))
 	continue;
 
       // consider only to be/remain installed rdeps
@@ -1110,6 +1216,35 @@ static bool or_group_subsumes(const pkgCache::DepIterator &d1,
 }
 
 
+/** Whether a particular version is security-related
+ *
+ * Useful e.g. to classify in menus as "Security Upgrade"
+ *
+ * @return \b true iff the given package version comes from security.d.o or
+ * known places
+ */
+bool is_security(const pkgCache::VerIterator &ver)
+{
+  static std::regex site_regex { "^security\\.(.+\\.)?debian.org$" };
+  std::smatch site_match;
+
+  for (pkgCache::VerFileIterator F = ver.FileList(); !F.end(); ++F)
+    {
+      pkgCache::PkgFileIterator fileit = F.File();
+      if (!fileit.end())
+	{
+	  string site  = fileit.Site()  ? fileit.Site()  : "";
+	  string label = fileit.Label() ? fileit.Label() : "";
+	  std::regex_search(site, site_match, site_regex);
+
+	  if (!site_match.empty() && label == "Debian-Security")
+	    return true;
+	}
+    }
+
+  return false;
+}
+
 // Interesting deps are:
 //
 //   - All critical deps
@@ -1220,6 +1355,63 @@ bool is_interesting_dep(const pkgCache::DepIterator &d,
     }
 }
 
+std::string get_uri(const pkgCache::VerIterator& ver,
+		    const pkgRecords* records)
+{
+  if (ver.end() || ver.FileList().end() || records == nullptr)
+    return string{};
+
+  for (auto vfi = ver.FileList(); !vfi.end(); ++vfi)
+    {
+      // match against source list
+      pkgIndexFile* index = nullptr;
+      if (apt_source_list->FindIndex(vfi.File(), index) == false)
+	continue;
+
+      // get package record
+      pkgRecords::Parser& parse = apt_package_records->Lookup(vfi);
+      if (_error->PendingError())
+	continue;
+
+      string pkg_file = parse.FileName();
+      if (pkg_file.empty())
+	continue;
+
+      string uri = index->ArchiveURI(pkg_file);
+
+      return uri;
+    }
+
+  return string{};
+}
+
+std::string get_origin(const pkgCache::VerIterator& ver,
+		       const pkgRecords* records)
+{
+  if (ver.end() || ver.FileList().end() || records == nullptr)
+    return string{};
+
+  if (ver.Downloadable())
+    {
+      return ver.RelStr();
+    }
+  else
+    {
+      return _("(installed locally)");
+    }
+}
+
+pkgCache::VerIterator get_candidate_version(const pkgCache::PkgIterator& pkg)
+{
+  if (apt_cache_file && ! pkg.end())
+    {
+      return (*apt_cache_file)[pkg].CandidateVerIter(*apt_cache_file);
+    }
+
+  // default return
+  return pkgCache::VerIterator();
+}
+
 std::wstring get_short_description(const pkgCache::VerIterator &ver,
 				   pkgRecords *records)
 {
@@ -1311,6 +1503,47 @@ namespace aptitude
 {
   namespace apt
   {
+    std::string priority_to_string(const pkgCache::State::VerPriority priority, bool short_form)
+    {
+      if (short_form)
+	{
+	  switch (priority)
+	    {
+	    case pkgCache::State::Important:
+	      // TRANSLATORS: Imp = Important
+	      return _("Imp");
+	    case pkgCache::State::Required:
+	      // TRANSLATORS: Req = Required
+	      return _("Req");
+	    case pkgCache::State::Standard:
+	      // TRANSLATORS: Std = Standard
+	      return _("Std");
+	    case pkgCache::State::Optional:
+	      // TRANSLATORS: Opt = Optional
+	      return _("Opt");
+	    case pkgCache::State::Extra:
+	      // TRANSLATORS: Xtr = Extra
+	      return _("Xtr");
+	    default:
+	      return _("ERR");
+	    }
+	}
+      else
+	{
+	  // this comes translated from apt
+	  const char* str = pkgCache::Priority(priority);
+	  if (strempty(str))
+	    {
+	      return _("ERROR");
+	    }
+	  else
+	    {
+	      return str;
+	    }
+	}
+    }
+
+
     bool is_full_replacement(const pkgCache::DepIterator &dep)
     {
       if(dep.end())
@@ -1385,5 +1618,85 @@ namespace aptitude
       const char *arch = ver.Arch();
       return apt_native_arch == arch || strcmp(arch, "all") == 0;
     }
+
+    bool clean_cache_dir()
+    {
+      string archivedir = aptcfg->FindDir("Dir::Cache::archives");
+
+      // lock the archive directory
+      FileFd lock;
+      if ( ! _config->FindB("Debug::NoLocking", false))
+	{
+	  lock.Fd(GetLock(archivedir + "lock"));
+	  if (_error->PendingError())
+	    {
+	      _error->Error(_("Unable to lock the download directory"));
+	      return false;
+	    }
+	}
+
+      // do clean
+      pkgAcquire fetcher;
+      fetcher.Clean(archivedir);
+      fetcher.Clean(archivedir+"partial/");
+
+      // return
+      if (_error->PendingError())
+	{
+	  return false;
+	}
+      else
+	{
+	  return true;
+	}
+    }
+
+
+    static std::unique_ptr<pkgAcquire_fetch_info> internal_fetchinfo {};
+
+    void reset_pkgAcquire_fetch_info()
+    {
+      internal_fetchinfo.reset();
+    }
+
+    void update_pkgAcquire_fetch_info()
+    {
+      if (!apt_cache_file || !apt_package_records)
+	return;
+
+      pkgAcquire fetcher;
+      pkgSourceList l;
+      if (!l.ReadMainList())
+	{
+	  _error->Error(_("Couldn't read list of sources"));
+	  return;
+	}
+
+      pkgDPkgPM pm(*apt_cache_file);
+      pm.GetArchives(&fetcher, &l, apt_package_records);
+      if (_error->PendingError())
+	return;
+
+      pkgAcquire_fetch_info f { fetcher.FetchNeeded(), fetcher.PartialPresent(), fetcher.TotalNeeded() };
+
+      internal_fetchinfo = std::make_unique<pkgAcquire_fetch_info>(f);
+    }
+
+    std::unique_ptr<pkgAcquire_fetch_info> get_pkgAcquire_fetch_info()
+    {
+      // if invalid, update and register for next time
+      if (!internal_fetchinfo && apt_cache_file)
+	{
+	  update_pkgAcquire_fetch_info();
+	  (*apt_cache_file)->package_state_changed.connect(sigc::ptr_fun(update_pkgAcquire_fetch_info));
+	  cache_closed.connect(sigc::ptr_fun(reset_pkgAcquire_fetch_info));
+	}
+
+      if (internal_fetchinfo)
+	return std::make_unique<pkgAcquire_fetch_info>(*internal_fetchinfo);
+      else
+	return {};
+    }
+
   }
 }
