@@ -1,7 +1,7 @@
 // aptcache.h  -*-c++-*-
 //
 //  Copyright 1999-2005, 2007-2009, 2011 Daniel Burrows
-//  Copyright 2015 Manuel A. Fernandez Montecelo
+//  Copyright 2015-2016 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,14 +15,16 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; see the file COPYING.  If not, write to
-//  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//  Boston, MA 02111-1307, USA.
+//  the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//  Boston, MA 02110-1301, USA.
 
 #ifndef APTCACHE_H
 #define APTCACHE_H
 
 #include <config.h>
 
+#include "usertags.h"
+
 #include <cwidget/generic/util/bool_accumulate.h>
 
 #include <apt-pkg/depcache.h>
@@ -36,6 +38,7 @@
 #include <set>
 #include <vector>
 
+
 /** \brief Replacements for the Apt cache and dependency cache file classes
  * 
  * 
@@ -58,39 +61,12 @@ template<typename PackageUniverse> class generic_solution;
 
 class aptitudeDepCache:public pkgDepCache, public sigc::trackable
 {
-  typedef int user_tag_reference;
-
 public:
   /** This is a general enum that's used for several purposes by the
    *  extended state.  Not every value is valid in every case.
    */
   enum changed_reason {manual, user_auto, libapt, from_resolver, unused};
 
-  /** \brief An opaque type used to store references into the
-   *  user-tags list.
-   *
-   *  We only store one copy of each tag string, to save space.
-   */
-  class user_tag
-  {
-    friend class aptitudeDepCache;
-    user_tag_reference tag_num;
-    explicit user_tag(const user_tag_reference &_tag_num) : tag_num(_tag_num)
-    {
-    }
-
-  public:
-    bool operator==(const user_tag &other) const
-    {
-      return tag_num == other.tag_num;
-    }
-
-    bool operator<(const user_tag &other) const
-    {
-      return tag_num < other.tag_num;
-    }
-  };
-
   /** This structure augments the basic depCache state structure to
    *  support special aptitude features.
    */
@@ -210,6 +186,9 @@ public:
    */
   bool read_only;
 
+  /** Collection of all user tags */
+  user_tag_collection user_tags;
+
   // Some internal classes for undo information
   class apt_undoer;
   class forget_undoer;
@@ -250,33 +229,8 @@ public:
     friend class aptitudeDepCache;
   };
 
-  const std::string &deref_user_tag(const user_tag &tag) const
-  {
-    return user_tags[tag.tag_num];
-  }
 private:
-  void parse_user_tags(std::set<user_tag> &tags,
-		       const char *&start, const char *end,
-		       const std::string &package_name);
-
   aptitude_state *package_states;
-  // To speed the program up and save memory, I only store one copy of
-  // each distinct tag, and keep a reference to this list.  The list
-  // is not managed especially intelligently: if you repeatedly add
-  // and remove never-before-seen tags to a package, it will grow
-  // without bound.  I don't consider this a problem, because users
-  // are unlikely to add "very many" (say, more than a few hundred)
-  // tags in a single session.  If it does become a problem, tags can
-  // be reference-counted, at the expense of maintaining an explicit
-  // free list.
-  std::vector<std::string> user_tags;
-  // Stores the reference corresponding to each string.
-  std::map<std::string, user_tag_reference> user_tags_index;
-  // Read a set of user tags from the given string region
-  // and write the tags into the index and into the given
-  // set of tags.
-  void parse_usertags(std::set<user_tag_reference> &tags,
-		      const char *&start, const char *end);
 
   int lock;
   // The lock on the extra-info file.
@@ -292,17 +246,6 @@ private:
 
   pkgRecords *records;
 
-  /** Call whenever the cache state is modified; discards the
-   *  state of the active resolver.
-   *
-   *  \param undo the undo group with which this discarding should be
-   *  associated.
-   */
-  void discard_resolver(undo_group *undo);
-
-  /** Call whenever a new resolver should be instantiated. */
-  void create_resolver();
-
   undoable *state_restorer(PkgIterator pkg, StateCache &state, aptitude_state &ext_state);
   // Returns an 'undoable' object which will restore the given package to the
   // given state via {Mark,Set}* routines
@@ -362,23 +305,32 @@ public:
   aptitudeDepCache(pkgCache *cache, Policy *Plcy=0);
 
   bool Init(OpProgress *Prog, bool WithLock,
-	    bool do_initselections, const char * status_fname=NULL);
+	    bool do_initselections,
+	    const char * status_fname,
+	    bool reset_reinstall);
 
-  bool is_locked() {return lock!=-1;}
+  bool is_locked() const { return (lock != -1); }
 
-  bool is_dirty() {return dirty;}
+  bool is_dirty() const { return dirty; }
 
   pkgRecords &get_records() { return *records; }
 
   // If do_initselections is "false", the "sticky states" will not be used
   // to initialize packages.  (important for the command-line mode)
-  bool build_selection_list(OpProgress &Prog, bool WithLock,
+  bool build_selection_list(OpProgress* Prog,
+			    bool WithLock,
 			    bool do_initselections,
-			    const char * status_fname=NULL);
+			    const char* status_fname,
+			    bool reset_reinstall);
 
+  /** Clears all information about which packages are 'new'.  Overwrites undoer
+   * if it is not NULL
+   */
   void forget_new(undoable **undoer);
-  // Clears all information about which packages are 'new'.  Overwrites undoer
-  // if it is not NULL
+  /** Clears all information about which packages are 'new'.  Overwrites undoer
+   * if it is not NULL
+   */
+  void forget_new(undoable **undoer, const std::vector<pkgCache::PkgIterator>& pkg_its);
 
   /** Sets the "new" flag for the given package; use this instead of
    *  directly manipulating the cache so the count of new packages is
@@ -392,7 +344,7 @@ public:
   inline aptitude_state &get_ext_state(const PkgIterator &Pkg)
   {return package_states[Pkg->ID];}
 
-  bool save_selection_list(OpProgress &prog, const char *status_fname=NULL);
+  bool save_selection_list(OpProgress* Prog, const char* status_fname = nullptr);
   // If the list isn't locked (or an fd isn't provided), is a NOP.
 
   void mark_install(const PkgIterator &Pkg, bool AutoInst, bool ReInstall, undo_group *undo);
@@ -451,12 +403,17 @@ public:
    */
   void get_upgradable(bool ignore_removed, std::set<pkgCache::PkgIterator> &upgradable);
 
+  /** Marks this package to be install, and all other packages to be kept.
+   *
+   *  The "keep" on the other packages, however, is NOT sticky and will NOT be
+   *  saved in the extended state file! (this is a bit of a hack..)  (this could
+   *  be fairly easily emulated; it's a convenience routine)
+   */
   void mark_single_install(const PkgIterator &pkg, undo_group *undo);
-  // Marks this package to be install, and all other packages to be kept.
-  //  The "keep" on the other packages, however, is NOT sticky and will NOT
-  // be saved in the extended state file! (this is a bit of a hack..)
-  // (this could be fairly easily emulated; it's a convenience routine)
 
+  /** Marks the given package as having been autoinstalled (so it will be
+   *  removed automatically) or having been installed manually.
+   */
   void mark_auto_installed(const PkgIterator &pkg,
 			   bool set_auto,
 			   undo_group *undo);
@@ -465,8 +422,10 @@ public:
    *
    *  The tag will be added to the user_tags member of the package's
    *  extended state.
+   *
+   * @return Whether the operation succeeded or not
    */
-  void attach_user_tag(const PkgIterator &pkg, const std::string &tag,
+  bool attach_user_tag(const PkgIterator &pkg, const std::string &tag,
 		       undo_group *undo);
 
   /** \brief Remove a tag from a package.
@@ -474,12 +433,17 @@ public:
    *  The tag will be removed from the user_tags member of the
    *  package's extended state.  If it isn't already present, nothing
    *  will happen.
+   *
+   * @return Whether the operation succeeded or not
    */
-  void detach_user_tag(const PkgIterator &pkg, const std::string &tag,
+  bool detach_user_tag(const PkgIterator &pkg, const std::string &tag,
 		       undo_group *undo);
 
-  // Marks the given package as having been autoinstalled (so it will be
-  // removed automatically) or having been installed manually.
+  /** Get user tags from a package
+   *
+   * @return The requested value
+   */
+  std::vector<std::string> get_user_tags(const PkgIterator& pkg);
 
   /** Retrieve the read-only flag. */
   bool get_read_only() const { return read_only; }
@@ -612,8 +576,11 @@ public:
   inline aptitudeDepCache::StateCache &operator [](pkgCache::PkgIterator const &I) {return (*DCache)[I];};
   inline unsigned char &operator [](pkgCache::DepIterator const &I) {return (*DCache)[I];};
 
-  bool Open(OpProgress &Progress, bool do_initselections, bool WithLock=true,
-	    const char * status_fname=NULL);
+  bool Open(OpProgress* Progress,
+	    bool do_initselections,
+	    bool WithLock,
+	    const char* status_fname,
+	    bool reset_reinstall);
   bool is_locked() {return have_system_lock;} // EWW (also not quite right)
 
   void ReleaseLock();
